# Program to take mbspect input parameters, run them through miriad and output measured parameters in an AGES-compatible format.
# See the user-input parameters section for requirements and usage.

import os
import string
from string import *
import re


# *** START OF USER-INPUT PARAMETERS ***
# Convert the FileList into a internal Python list
List = open('FileList.txt','r')

# Set whether to insist on only operating within the velocity range of the cube. If True, will truncate the xrange to stay withing the available
# data. If False, this will have no ill effects except that the amount of spectral baseline may be asymmetrical. Will only fail if the velocity
# range requested is entirely outside the available data.
usecubevel = True

# Set minimum and maximum velocity range of the cube. Ignored if usecubevel is False.
minxr = 100.0
maxxr = 20000.0

# Set the velocity range (centred on the source) used for calculating the baseline polynomial. Total velocity range will be twice this value.
velbaserange = 2000.0

# Set the name of the miriad file to use. If set to None, will use the file specified in each input file.
mirfile = None

# *** END OF USER-INPUT PARAMETERS ***


Files=[]

for line in List:
	# Splitting by \n removes line breaks
	Files.append(line.split('\n')[0])

List.close()

for j in range(0,len(Files)):
	commands=[]		# This will be a string array containing all the input parameters

	Input = open(Files[j],'r')
	print('Processing ',Files[j])

	# We want to manually set the filename and set the xrange based on the profile
	# So, first go through file to get profile parameters
	for line in Input:	
		if 'profile=' in line:
			# First split by the comma to get profile=vmin
			half = line.split(',')[0]
			# Now split by equals to get vmin itself
			vmin = half.split('=')[1]
			# Can get vmax directly
			vmax = line.split(',')[1]
			
			vcent = (float(vmax) + float(vmin)) / 2.0
	
	Input.seek(0)	
	
	# Now, having re-wound the file, we do it again to set the mbspect input commands
	for line in Input:
		# Skip commented lines
		if '#' in line:
			continue
		
		# Set an identical copy so we still have access to the original	
		thisline = line
		
		# Some lines may need special modifications.	
		# 1) If the user has specified a miriad file, replace the "in=" line to contain the name of this file. Otherwise it will
		# just use the line unaltered, i.e. use the file specified in each input file.
		if mirfile is not None:
			if 'in=' in thisline:
				thisline = str('in='+mirfile)+'\n'
			
		# 2) Set xrange based on vcent
		if 'xrange=' in thisline:
			xmin = vcent - velbaserange
			xmax = vcent + velbaserange

			# If the user insists on only using the velocities within the actual data, truncate the xrange if necessary
			if usecubevel == True:
				if xmin < minxr:
					xmin = minxr

				if xmax > maxxr:
					xmax = maxxr

			thisline = 'xrange='+str(xmin)+','+str(xmax)+'\n'
			
		# 3) Set the device to be postscript
		if 'device=' in thisline:
			thisline = 'device='+str(Files[j]).split('_mbspect.txt')[0]+'.ps/PS'		
							
		# Additional - Save the name of the log file as it will be needed later
		if 'log' in line:
			logfile = line.split('=')[1]
			# Annoyingly, have to use weired regular expressions to
			# remove all whitespace
			logfile = re.sub(r'\s', '',logfile)
		
		# 4) Enclose the comment in single quote marks
		if 'comment=' in line:
			linesection = line.replace('\n','').split('=')[1]
			thisline = 'comment=\''+linesection.replace('\'','').replace('\"','')+'\''+'\n'

		# Now we can add the line to the command list
		commands.append(thisline)

	Input.close()

	comstring = 'mbspect '
	
	# Get all the input parameters into a single, one-line string
	for i in range(0,len(commands)):
		# Annoying regular expressions syntax to remove all whitespace
		comstring = comstring + re.sub(r'\s', '',commands[i])	+ ' '

	# Write this string into a file miriad can accept as a command	
	ComFile = open('MiriadInputs','w')

	ComFile.write(comstring+'\n')
	ComFile.write('exit')

	ComFile.close()

	# Run mbspect with the command file just produced, save the output. Note that the usual command :
	#os.system('miriad < MiriadInputs > TempOutput.txt')
	# - does not appear to work on Windows Subsystem for Linux. However the following replacement does
	# work, and shoudld work equally well on standard Linux.
	os.system('./MiriadInputs > TempOutput.txt')

	os.system('rm MiriadInputs')

	# Now we have all 3 files, but the inputs and mbspect output need to be properly formatted.
	# First produce a commented version of the input file.

	Temp1 = open('TempInputs.txt','w')
	Temp1.write('# INPUT PARAMETERS'+'\n')

	for i in range(0,len(commands)):
		thing = str('#') + str(commands[i])
		Temp1.write(thing)

	Temp1.close()

	# Next, trim and comment the msbpect outputs
	Output = open('TempOutput.txt','r')
	FormatOutput = open('TempOutput2.txt','w')

	i = -1

	# If position fitting fails there will be nothing in the TempOutput file apart
	# from the input parameters. So count number of actual output lines.
	usefullines = 0
	for line in Output:	
		i = i + 1
		
		# Skip the first 4 lines, and omit the final line which tells miriad
		# to exit.
		if (i <=3 or ('exit' in line)) and ('#' not in line):
			pass
		else:
			usefullines = usefullines + 1
			if line.startswith('#'):
				FormatOutput.write(line)
			else:
				thing = str('#')+str(line)
				FormatOutput.write(thing)
				
	# If there was no output, inform the user and call a halt before any files have
	# been altered (except the Temp files, which can safely be removed)
	if usefullines	== 0:
		print('Error running file ',Files[j])
		print('Program halted, nothing altered. Correct the file and try again.')
		print('Also remove all files before this one in FileList !')	

	FormatOutput.close()

	# Now everything is properly formatted. Just need to join everything together !

	Complete=[]
	FirstFile = open('TempInputs.txt','r')
	SecondFile= open('TempOutput2.txt','r')
	ThirdFile =open(logfile,'r')

	for line in FirstFile:
		Complete.append(line)
	for line in SecondFile:
		Complete.append(line)
	for line in ThirdFile:
		Complete.append(line)

	FirstFile.close()
	SecondFile.close()
	ThirdFile.close()
	

	filename = logfile.split('.txt')[0]+str('spec')+str('.txt')

	Final = open(filename,'w')

	for i in range(0,len(Complete)):
		Final.write(Complete[i])

	Final.close()

	os.system('rm Temp*')
	
	# Remove the input files and log files produced.
	os.remove(Files[j])
	try:
		os.remove(logfile)
	except:
		pass
